'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

var _ChargeV = require('./ChargeV2.js');

var _ChargeV2 = _interopRequireDefault(_ChargeV);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = {

    /**
     * @return The orderCharges (ChargesV2) that should be added to the order
     */
    getOrderCharges: function getOrderCharges(_ref) {
        var order = _ref.order,
            chargesV2 = _ref.chargesV2,
            timezone = _ref.timezone;


        var orderCharges = [];
        var prevOrderCharges = [];

        var deliveryTime = _lodash2.default.get(order, 'delivery.time') || (0, _moment2.default)();
        if (timezone && _lodash2.default.isNumber(deliveryTime)) {
            deliveryTime = (0, _moment2.default)(deliveryTime).tz(timezone);
        }

        do {
            prevOrderCharges = orderCharges;

            orderCharges = _lodash2.default.compact(_lodash2.default.map(chargesV2, function (charge) {

                var isApplicable = _ChargeV2.default.isApplicable({
                    charge: charge,
                    deliveryTime: deliveryTime,
                    deliveryType: order.delivery.type,
                    orderItems: order.orderItems,
                    source: order.source,
                    platform: order.platform
                });

                if (!isApplicable) {
                    if (charge.mandatory) {
                        return { chargeId: charge.id, amount: 0 };
                    } else {
                        return null;
                    }
                } else {
                    return {
                        chargeId: charge.id,
                        amount: _ChargeV2.default.calculateAmount({
                            charge: charge,
                            deliveryTime: deliveryTime,
                            deliveryType: order.delivery.type,
                            source: order.source,
                            platform: order.platform,
                            orderItems: order.orderItems,
                            orderCharges: orderCharges
                        })
                    };
                }
            }));

            orderCharges = _lodash2.default.sortBy(orderCharges, 'chargeId');
            prevOrderCharges = _lodash2.default.sortBy(prevOrderCharges, 'chargeId');
        } while (!_lodash2.default.isEqual(orderCharges, prevOrderCharges));

        return orderCharges;
    }
};